/*
 * config.h
 *
 *  Created on: Apr 26, 2011
 *      Author: tony
 */


/*
    License

    This file is part of ipclassify.

    ipclassify is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ipclassify is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ipclassify.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CONFIG_H_
#define CONFIG_H_

/* ----- Includes ----------------------------------------------------- */

#include <syslog.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <rb.h>
#include <limits.h>
#include <netinet/in.h>
#include <linux/netfilter.h>
#include <libnetfilter_queue/libnetfilter_queue.h>
#include <linux/netfilter_ipv4.h>
#include "basic.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

/* -------------------------------------------------------------------- */


/* ----- Defines ------------------------------------------------------ */
/* Used in the RBT Keys to store two unsigned longs */
#define LOWMASK  0x00000000FFFFFFFF
#define HIGHMASK 0xFFFFFFFF00000000
#define ZEROMASK 0x0000000000000000
/*
 *  To put an uint in the high bits:
 *    int = integer;
 *    int = int<<32;
 *  To put an uint in the Low bits:
 *    int = 0;
 *    int = integer;
 *
 *  To get uint out of the low bits:
 *    integer = LOWMASK&int;
 *  To get uint out of the High bits:
 *    integer = HIGHMASK&int;
 *    integer = integer>>32;
 */
#define gethigh(a)	((HIGHMASK & a) >> 32)
#define getlow(a)	(LOWMASK & a)
#define sethigh(a)	(a << 32)
#define setlow(a, b)	(a | (b & LOWMASK))

/* Default Buffer Size */
#define DEFBUFSIZE 256

/* Default network Buffer Size */
#define BUFSIZE   2048
#define PAYLOADSIZE 21

#define SRC_ADDR(payload) (*(in_addr_t *)((payload)+12))
#define DEST_ADDR(payload) (*(in_addr_t *)((payload)+16))


typedef enum {
	STATUS_OK = 0,
	MERGED,
	SKIPPED,
	NOTFOUND,
	STATUS_NOOK,
}               CallbackStatus_t;


/* ----- Structures --------------------------------------------------- */


typedef struct {
	unsigned int queue;
	cp_rbtree *rbt;
	cp_rbtree *whitelist;
	struct queue_t *next;
	struct nfq_q_handle *qh;
	unsigned long long ttl_packets;
	unsigned long long ttl_rejected;
	pthread_rwlock_t rwlock;
	int acceptMark;
	int rejectMark;
} queue_t;

/*
 * Range is the real key which is ipmax^ipmin
 * RangeCheck is either true if we want the comparison function
 *   to compare against the range or
 * False if we want an absolute check.
 */
typedef struct {
	unsigned long long key;
	int RangeCheck;
} keyType;

/* RBT definition (key is ipmax^ipmin) */
typedef struct {
    char *blockname;                  /* data */
    unsigned long hits;               /* How many hits on this rule */
    unsigned long long key;
} recType;


/* ----- Prototypes --------------------------------------------------- */

void Terminate(int);
void SigHandle(int);
int  checkPID (void);
void usage    (void);
void debugMsg (char *, int);
void log2file (char *);
void log2stats(char *);
void loadfile (int, char *,int);

/*
 * sets the accept or reject mark.
 * args are: TRUE for set accept mark, FALSE for set reject mark
 *           queue
 *           mark
 */
void setMark(int, int, int);

/* The worker threads */
void *worker_thread_loop(void *);
static int nfqueue_cb(struct nfq_q_handle *, struct nfgenmsg *,
            struct nfq_data *, void *);
char *ip2str(in_addr_t);
void netlink_init(void);

/* the following are needed by cscope rbt */
int  compKey(keyType *, keyType *);
keyType *copyKey(keyType *);
recType *copyValue(recType *);
void destroyValue(recType *);

/* add the (int) queue the (char *)basename/ipmin/ipmax, if whitelist is TRUE add to the whitelist */
void ranged_insert(int, char *, char *, char *, int);
/* find the queue on the threadlist, return NULL if not there */
queue_t *findQueue(int);
/* Create new queue int and return pointer to new node */
queue_t *addQueue(int);

/*
 *  for the given queue_t node, add the range indicated by:
 *   name/ipmin/ipmax. if whitelist is TRUE this is a whitelist
 */
int addToRBT(queue_t *,char *,char *,char *, int);

int findInRBT(queue_t *,unsigned long long, recType *);

int RBTDumpStats(queue_t *,unsigned int);
int RBTResetStats(queue_t *);

int statsDumpCallback(void *,void *);
int statsResetCallback(void *,void *);

#endif /* CONFIG_H_ */
