

%{
/*
    This file is part of ipclassify.

    ipclassify is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    ipclassify is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ipclassify.  If not, see <http://www.gnu.org/licenses/>.
*/

  /* Definition Section of lexer */
extern void Terminate(int);
extern void debugMsg (char *, int);
extern void log2file(char *);

extern char *FileName;

 /* Includes */
#include "parser.h"
 
#ifdef DMALLOC
#include "dmalloc.h"
#endif

%}

%option yylineno

%x COMMENT
 
%%
  /* Rules Section */

  /* Consume all comments */
#.*\n ;

 /* ignore white space */
[ \t\n]+ ;

[0-9]+  { yylval.intval = atoi(yytext); return NUMBER;}

ACCEPT |
accept |
Accept	{ return ACCEPTMARK; }

REJECT |
reject |
Reject { return REJECTMARK; }

MARK |
mark |
Mark   { return MARK; }

WHITELIST |
whitelist |
Whitelist	{ return WHITELIST; }

ALL |
All |
all { return ALL; }

[A-Za-z0-9\/\.\_]*  {
    yylval.textval = yytext;
    if(FileName != NULL) { free(FileName); } 
    FileName = strdup(yytext); 
    return(FILENAME); }

 /* Self-Contained Comment */
^[ \t]*"/*".*"*/"[ \t]*\n ;

[ \t]*"/*" {BEGIN COMMENT;}

 /*
  * The Comment Start State \
  */
<COMMENT>{

"*/"  {BEGIN 0;}

 /* ignore white space */
[ \t\n] ;

[^("*/)]+ ;

.+"*/"[ \t]*\n {BEGIN 0;}

}

 /* default rule */
<*>. return yytext[0];

%%


yyerror(const char *msg) {
  char mstr[256];
  
  snprintf((char *)mstr, 256, "yyerror(): line %d: %s at '%s'",
			yylineno, msg, yytext);
  log2file(mstr);
  debugMsg(mstr,1);
  Terminate(1);
}


 
